/** 
 * @class hedge::tDataObject
 *  
 * @brief A class  implementing a reference counter for  object of the
 * DCEL data structure.
 *
 * @details All classes that are suposed to use reference counting must
 *          inherit from this class.
 *
 * @author
 * Marcelo Ferreira Siqueira @n
 * Universidade Federal de Mato Grosso do Sul, @n
 * Departamento de Computacao e Estatistica, @n
 * marcelo at dct (dot) ufms (dot) br
 *
 * @version 1.0
 * @date Abril 2008
 *
 * @attention This program is distributed  WITHOUT ANY WARRANTY,  and
 *            it may be freely redistributed under the condition that
 *            the copyright notices are  not  removed,  and  no
 *            compensation  is  received. Private, research, and
 *            institutional use  is free. Distribution of this code as
 *            part of a commercial system is permissible ONLY BY DIRECT
 *            ARRANGEMENT WITH THE AUTHOR.
 */

#ifndef DATAOBJ_H_
#define DATAOBJ_H_


#include <iostream>


namespace dcel {


  /**
   * Class tDataObject: implements a reference counter.
   *
   */ 
  class tDataObject {
  public:
    /**
     * Constructs a data object with zero references.
     */
    tDataObject() : m_refcount(0) {}

    
    /**
     * Virtual Destructor.
     */
    virtual ~tDataObject() {
      if (m_refcount > 0) {
	std::cout << "Trying to delete object with non-zero reference count"
                  << std::endl;
      }
    }


    /**
     * Increases the counter of references.
     */
    void add_ref() {
      ++m_refcount;
    }


    /**
     * Decreases the counter of references and destroy the object if it
     * reaches zero.
     *
     */
    void release() {
      --m_refcount;
      if (m_refcount == 0) delete this;
    }


    /**
     * Returns the number of references to this object.
     *
     * @return The number of references to this object.
     *
     */
    int get_ref() const { return m_refcount; }


  protected:  
    /**
     * Counter for  the number  of references to  this object.   It is
     * "mutable" to  allow the reference counter  of derived, constant
     * objects to updated.
     */
    mutable int m_refcount;
  };


}

#endif  // DATAOBJ_H_
