/** 
 * @class dcel::tVTAttrib
 *  
 * @brief A class representing a  set of vertex attributes.
 *
 * @author
 * Marcelo Ferreira Siqueira @n
 * Universidade Federal de Mato Grosso do Sul, @n
 * Departamento de Computacao e Estatistica, @n
 * marcelo at dct (dot) ufms (dot) br
 *
 * @version 1.0
 * @date Abril 2008
 *
 * @attention This program is distributed  WITHOUT ANY WARRANTY,  and
 *            it may be freely redistributed under the condition that
 *            the copyright notices are  not  removed,  and  no
 *            compensation  is  received. Private, research, and
 *            institutional use  is free. Distribution of this code as
 *            part of a commercial system is permissible ONLY BY DIRECT
 *            ARRANGEMENT WITH THE AUTHOR.
 */


#ifndef VTATTRIB_H_
#define VTATTRIB_H_


#include "smartptr.h"
#include "dataobj.h"


namespace dcel {


  /**
   * Forward definition of classes.
   */

  class tComplex;
  class tVertex;

  
  class tVTAttrib : public tDataObject {
  public:
    /**
     * Destructor.
     */
    virtual ~tVTAttrib() {}


    /**
     * Returns a pointer to the owner of this attribute.
     *
     * @return A pointer to a vertex.
     * 
     */
    tSmartPtr<tVertex> get_owner() const {
      return m_owner;
    }


    /**
     * Makes tComplex a friend class.
     *
     */
    friend class tComplex;


  protected:
    /**
     * Default constructor.
     */
    tVTAttrib() {}


    /**
     * Assigns a given pointer to the owner attribute of this class.
     *
     * @param v: pointer to a vertex.
     * 
     */
    void set_owner(const tSmartPtr<tVertex>& v) {
      m_owner = v;
    }


    /** 
     * Pointer to the owner of this attribute. 
     */
    tSmartPtr<tVertex> m_owner;
    
  };


}


#endif   // VTATTRIB_H_
