//  ===================================================================
//
//  Program:   common
//  Module:    mathmacros.h
//  Language:  C++
//  Date:      June 2004
//  Version:   1.0.0
//
//  Copyright (c) 2004
//
//  Marcelo Siqueira
//
//  Graduate Student
//  Department of Computer and Information Science
//  Moore School Building
//  200 South 33rd Street, Suite #62
//  Philadelphia, PA 19104-6389
//  Phone No.: (215) 898-5879
//
//  marcelos@seas.upenn.edu
//
// This program  is distributed  WITHOUT ANY WARRANTY,  and it  may be
// freely redistributed under the condition that the copyright notices
// are  not  removed,  and  no  compensation  is  received.   Private,
// research, and institutional use  is free. Distribution of this code
// as  part of  a  commercial  system is  permissible  ONLY BY  DIRECT
// ARRANGEMENT WITH THE AUTHOR.
//
//  ==================================================================

#ifndef __mathmacros_h_
#define __mathmacros_h_

#include <cmath>

#define PI	         3.14159265
#define D_HUGE		 1e40
#define SQR(x)		 ((x) * (x))
#define CUBE(x)		 ((x) * (x) * (x))
#define INVERSE(x)       (1. / (x))
#define MID(x)		 ((x) / 2.)
#define DUP(x)		 ((x) * 2.)
#define ABS(x)		 (fabs(x))
#define MOD(x)		 (ABS(x))
#define ISZERO(x)	 (ABS(x) <= 1.e-20)
#define ISNEGATIVE(x)	 ((x) < 0.)
#define ISPOSITIVE(x)	 ((x) > 0.)
#define ISEQUAL(x,y)	 (ISZERO((x) - (y)))
#define IS2PI(x)	 (ISEQUAL((x), 2 * PI))
#define MIN(x,y)	 ((x) < (y) ? (x) : (y))
#define MAX(x,y)	 ((x) > (y) ? (x) : (y))
#define TORADIANS(x)	 ((x) * PI / 180.)
#define TODEGREES(x)	 ((x) * 180. / PI)
#define SIGN(x)		 (ISNEGATIVE(x) ? -1 : 1)

  //
  // Compare functions templates
  //
  template <class T> inline bool
  isEqual(T a, T b) {
    return a == b;
  }
  
  template <class T> inline bool
  isZero(T a) {
    return a == 0;
  }
  
  template <class T> inline bool
  isPositive(T a) {
    return a > 0;
  }
  
  template <class T> inline bool
  isNegative(T a) {
    return a < 0;
  }
  
  //
  // Override compare functions templates for double type
  //
  inline bool
  isEqual(double a, double b) {
    return ISEQUAL(a, b);
  }

  inline bool
  isZero(double a) {
    return ISZERO(a);
  }
  
  inline bool
  isPositive(double a) {
    return a > 0 && !isZero(a);
  }
  
  inline bool
  isNegative(double a) {
    return a < 0 && !isZero(a);
  }

  //
  // Other compare functions for double type
  //
  inline bool
  isLesser(double a, double b) {
    return a < b && !isEqual(a, b);
  }

  inline bool
  isLesserEqual(double a, double b) {
    return a < b || isEqual(a, b);
  }

  inline bool
  isGreater(double a, double b) {
    return a > b && !isEqual(a, b);
  }

  inline bool
  isGreaterEqual(double a, double b) {
    return a > b || isEqual(a, b);
  }

  //
  // Other functions
  //
  inline double
  toRadians(double a) {
    return TORADIANS(a);
  }

  inline double
  toDegrees(double a) {
    return TODEGREES(a);
  }

  inline double
  sqr(double a) {
    return a * a;
  }

  inline double
  mysqrt(double a) {
    if (a < 0) return 0;
    return sqrt(a);
  }


#endif	 // __mathmacros_h_
