//  ===================================================================
//
//  Program:  OFFLIB
//  Module:   lexer.cpp
//  Language: C++
//  Date:     February 2008
//  Version:  1.0.0
//
//  Copyright (c) 2008
//
//  Marcelo Siqueira
//
//  Departamento de Computao e Estatstica
//  Universidade Federal de Mato Grosso do Sul
//  Av. Costa e Silva, S/N, Campus Universitria
//  Campo Grande, MS, Brasil, 79090-700
//  Phone No.: +55 67 3345 7458
//             +55 67 3345 7515
//
//  marcelo@dct.ufms.br
//
// This program  is distributed  WITHOUT ANY WARRANTY,  and it  may be
// freely redistributed under the condition that the copyright notices
// are  not  removed,  and  no  compensation  is  received.   Private,
// research, and institutional use  is free. Distribution of this code
// as  part of  a  commercial  system is  permissible  ONLY BY  DIRECT
// ARRANGEMENT WITH THE AUTHOR.
//
//  ===================================================================

#include "lexer.h"

#include <sstream>
#include <cctype>
#include <stdlib.h>


namespace offlib {


  // -----------------------------------------------------------------
  // Method: tLexer()
  // -----------------------------------------------------------------
  tLexer::tLexer (std::istream& is) : m_realstr(is)
  {
    m_lcount = 0;
    m_ccount = 0;
  }


  // -----------------------------------------------------------------
  // Method: get_str()
  // -----------------------------------------------------------------
  bool
  tLexer::get_str(std::string& s) throw (tExceptionObject)
  {
    if (!skip_space()) {
      return false;
    }

    char c;
    if (!m_realstr.get(c)) {
      return false;
    }

    if (isalpha(c)) {
      char str[256];
      int i = 0;
      do {
        str[i] = c;

        if (!m_realstr.get(c)) {
          return false;
        }

        increment_char();

        ++i;
      } while (isalnum(c) && (i < 255));

      str[i] = '\0';

      if (isalnum(c)) {
        std::stringstream ss(std::stringstream::in | 
                             std::stringstream::out);
        ss << "String is too big";
        throw tExceptionObject(__FILE__,__LINE__,ss.str());
      }

      s = str;
      m_realstr.putback(c);
    }
    else {
      m_realstr.putback(c);
      return false;
    }

    return true;
  }


  // -----------------------------------------------------------------
  // Method: get_int()
  // -----------------------------------------------------------------
  bool
  tLexer::get_int(int& x) throw (tExceptionObject)
  {
    if (!skip_space()) {
      return false;
    }

    char c;
    if (!m_realstr.get(c)) {
      return false;
    }

    if (isdigit(c)) {
      char str[256];
      int i = 0;
      do {
        str[i] = c;

        if (!m_realstr.get(c)) {
          return false;
        }

        increment_char();

        ++i;
      } while (isdigit(c) && (i < 255));

      str[i] = '\0';

      if (isdigit(c)) {
        std::stringstream ss(std::stringstream::in | 
                             std::stringstream::out);
        ss << "Number is too big";
        throw tExceptionObject(__FILE__,__LINE__,ss.str());
      }

      x = atoi(str);
      m_realstr.putback(c);
    }
    else {
      m_realstr.putback(c);
      return false;
    }

    return true;
  }


  // -----------------------------------------------------------------
  // Method: get_uint()
  // -----------------------------------------------------------------
  bool
  tLexer::get_uint(unsigned& x) throw (tExceptionObject)
  {
    if (!skip_space()) {
      return false;
    }

    char c;
    if (!m_realstr.get(c)) {
      return false;
    }

    if (isdigit(c)) {
      char str[256];
      int i = 0;
      do {
        str[i] = c;

        if (!m_realstr.get(c)) {
          return false;
        }

        increment_char();

        ++i;
      } while (isdigit(c) && (i < 255));

      str[i] = '\0';

      if (isdigit(c)) {
        std::stringstream ss(std::stringstream::in | 
                             std::stringstream::out);
        ss << "Number is too big";
        throw tExceptionObject(__FILE__,__LINE__,ss.str());
      }

      x = unsigned(atoi(str));
      m_realstr.putback(c);
    }
    else {
      m_realstr.putback(c);
      return false;
    }

    return true;
  }


  // -----------------------------------------------------------------
  // Method: get_double()
  // -----------------------------------------------------------------
  bool
  tLexer::get_double(double& d) throw (tExceptionObject)
  {
    if (!skip_space()) {
      return false;
    }

    char c;
    if (!m_realstr.get(c)) {
      return false;
    }
   
    m_realstr.putback(c);
    if (!(m_realstr >> d)) {
      std::stringstream ss(std::stringstream::in | 
                           std::stringstream::out);
      ss << "Error while reading double";
      throw tExceptionObject(__FILE__,__LINE__,ss.str());
    }

    return true;
  }


  // -----------------------------------------------------------------
  // Method: increment_line()
  // -----------------------------------------------------------------
  void
  tLexer::increment_line()
  {
    ++m_lcount;
    reset_char_counter();
  }


  // -----------------------------------------------------------------
  // Method: increment_char()
  // -----------------------------------------------------------------
  void
  tLexer::increment_char()
  {
    ++m_ccount;
  }


  // -----------------------------------------------------------------
  // Method: decrement_char()
  // -----------------------------------------------------------------
  void
  tLexer::decrement_char() 
  {
    if (m_ccount > 0) {
      --m_ccount;
    }
  }


  // -----------------------------------------------------------------
  // Method: skip_space()
  // -----------------------------------------------------------------
  bool
  tLexer::skip_space() 
  {
    char c;
    for (;;) {
      if (!m_realstr.get(c)) {
        return false;
      }

      increment_char();

      if (c == '\n') {
        increment_line();
      }

      if (!isspace(c)) {
        m_realstr.putback(c);
        decrement_char();
        return true;
      }
    }

    return true;
  }

}        // end of namespace offlib
