/** 
 * @class dcel::tMyFCAttrib
 *  
 * @brief A class representing a set of face attributes.
 *
 * @author
 * Marcelo Ferreira Siqueira @n
 * Universidade Federal de Mato Grosso do Sul, @n
 * Departamento de Computacao e Estatistica, @n
 * marcelo at dct (dot) ufms (dot) br
 *
 * @version 1.0
 * @date November 2008
 *
 * @attention This program is distributed  WITHOUT ANY WARRANTY,  and
 *            it may be freely redistributed under the condition that
 *            the copyright notices are  not  removed,  and  no
 *            compensation  is  received. Private, research, and
 *            institutional use  is free. Distribution of this code as
 *            part of a commercial system is permissible ONLY BY DIRECT
 *            ARRANGEMENT WITH THE AUTHOR.
 */

#include "myfcattrib.h"


namespace pps {


  /**
   * Destructor.
   *
   */
  tMyFCAttrib::~tMyFCAttrib()
  {
    if (m_c != 0) delete[] m_c;
  }


  /**
   * Returns the i-th projected  point of the surface patch associated
   * with the face that owns this attribute.
   *
   * @param i: the index of the point to be returned.
   * @param pt: pointer to the point to be returned. 
   *
   * @return The i-th projected  point of the surface patch associated
   * with the face that owns this attribute.
   *
   */
   void
   tMyFCAttrib::get_point( unsigned i, double* pt ) const 
     throw (tExceptionObject)
   {
     const unsigned K = m_n + 6;

     if ( i >= K ) {
       std::stringstream ss(std::stringstream::in | 
                            std::stringstream::out);
       ss << "Index out of range";
       throw tExceptionObject(__FILE__,__LINE__,ss.str());
     }

     pt[ index(0,i,K) ] = m_c[ index(0,i,K) ];
     pt[ index(1,i,K) ] = m_c[ index(1,i,K) ];
     pt[ index(2,i,K) ] = m_c[ index(2,i,K) ];

     return;
   }

}
