/** 
 * @class dcel::tMyVTAttrib
 *  
 * @brief A class representing a set of vertex attributes.
 *
 * @author
 * Marcelo Ferreira Siqueira @n
 * Universidade Federal de Mato Grosso do Sul, @n
 * Departamento de Computacao e Estatistica, @n
 * marcelo at dct (dot) ufms (dot) br
 *
 * @version 1.0
 * @date Abril 2008
 *
 * @attention This program is distributed  WITHOUT ANY WARRANTY,  and
 *            it may be freely redistributed under the condition that
 *            the copyright notices are  not  removed,  and  no
 *            compensation  is  received. Private, research, and
 *            institutional use  is free. Distribution of this code as
 *            part of a commercial system is permissible ONLY BY DIRECT
 *            ARRANGEMENT WITH THE AUTHOR.
 */

#include "myvtattrib.h"

#include "dcel.h"

#include "mathmacros.h"

#include <cassert>


namespace pps {


  /**
   * Computes the degree of the vertex that owns this attribute.
   *
   */
  void
  tMyVTAttrib::compute_degree()
  {
    vtPointer vt = this->get_owner();

    assert( vt.is_not_null() );

    hePointer he1 = vt->get_hedge();

    assert( he1.is_not_null() );

    hePointer he2 = MATE( he1->prev() );

    unsigned d = 1;

    do {

      assert( he2.is_not_null() );

      ++d;

      he2 = MATE( he2->prev() );
    }
    while (he1 != he2);

    this->set_degree(d);
  }

}
